@@ -63,12 +63,17 @@
 #include <netinet/in.h>
 #include <netinet/tcp.h>
 
-#include <OpenIPMI/serv.h>
-#include <OpenIPMI/mcserv.h>
 #include <OpenIPMI/lanserv.h>
+#include <OpenIPMI/mcserv.h>
 #include <OpenIPMI/ipmi_lan.h>
 #include <OpenIPMI/ipmi_err.h>
 #include <OpenIPMI/persist.h>
+#include <OpenIPMI/internal/winsock_compat.h>
+#include "ipmi_sim.h"
+#include "sol.h"
+#include "emu.h"
+
+static int sol_init_mc(sys_data_t *sys, lmc_data_t *mc);
 
 /* FIXME - move to configure handling */
 #define USE_UUCP_LOCKING
@@ -575,7 +580,7 @@ sol_tcp_shutdown(ipmi_sol_t *sol)
     soldata_t *sd = sol->soldata;
 
     if (sd->fd >= 0)
-	close(sd->fd);
+	close_socket(sd->fd);
     sd->fd = -1;
 }
 
@@ -586,11 +591,12 @@ sol_tcp_initialize(ipmi_sol_t *sol)
     struct addrinfo hints, *addr;
     int rv;
     int options;
+    sys_data_t *sys = sd->sys;
 
     if (sd->fd != -1) {
-	sd->logchan->log(sd->logchan, OS_ERROR, NULL,
-			 "Error sol activate on active port %s:%s",
-			 sol->tcpdest, sol->tcpport);
+	sys->log(sys, OS_ERROR, NULL,
+		 "Error sol activate on active port %s:%s",
+		 sol->tcpdest, sol->tcpport);
 	return -1;
     }
 	
@@ -599,29 +605,29 @@ sol_tcp_initialize(ipmi_sol_t *sol)
     hints.ai_socktype = SOCK_STREAM;
     rv = getaddrinfo(sol->tcpdest, sol->tcpport, &hints, &addr);
     if (rv != 0) {
-	sd->logchan->log(sd->logchan, OS_ERROR, NULL,
-			 "Error getting tcp sol port address for %s:%s: %s",
-			 sol->tcpdest, sol->tcpport, gai_strerror(rv));
+	sys->log(sys, OS_ERROR, NULL,
+		 "Error getting tcp sol port address for %s:%s: %s",
+		 sol->tcpdest, sol->tcpport, gai_strerror(rv));
 	return -1;
     }
 
     sd->fd = socket(addr->ai_family, SOCK_STREAM, 0);
     if (sd->fd == -1) {
 	rv = -1;
-	sd->logchan->log(sd->logchan, OS_ERROR, NULL,
-			 "Error creating tcp sol port socket for %s:%s: %s",
-			 sol->tcpdest, sol->tcpport, strerror(errno));
+	sys->log(sys, OS_ERROR, NULL,
+		 "Error creating tcp sol port socket for %s:%s: %s",
+		 sol->tcpdest, sol->tcpport, strerror(errno));
 	goto out;
     }
 
     rv = connect(sd->fd, addr->ai_addr, addr->ai_addrlen);
     if (rv == -1) {
-	close(sd->fd);
+	close_socket(sd->fd);
 	sd->fd = -1;
-	if (sd->sys->debug & DEBUG_SOL)
-	    sd->logchan->log(sd->logchan, OS_ERROR, NULL,
-			   "Error connecting tcp sol port socket for %s:%s: %s",
-			   sol->tcpdest, sol->tcpport, strerror(errno));
+	if (sys->debug & DEBUG_SOL)
+	    sys->log(sys, OS_ERROR, NULL,
+		     "Error connecting tcp sol port socket for %s:%s: %s",
+		     sol->tcpdest, sol->tcpport, strerror(errno));
 	goto out;
     }
 
@@ -629,23 +635,23 @@ sol_tcp_initialize(ipmi_sol_t *sol)
     rv = setsockopt(sd->fd, IPPROTO_TCP, TCP_NODELAY,
 		    (char *) &options, sizeof(options));
     if (rv == -1) {
-	close(sd->fd);
+	close_socket(sd->fd);
 	sd->fd = -1;
-	sd->logchan->log(sd->logchan, OS_ERROR, NULL,
-			 "Error setting nodelay on tcp sol port socket"
-			 " for %s:%s: %s",
-			 sol->tcpdest, sol->tcpport, strerror(errno));
+	sys->log(sys, OS_ERROR, NULL,
+		 "Error setting nodelay on tcp sol port socket"
+		 " for %s:%s: %s",
+		 sol->tcpdest, sol->tcpport, strerror(errno));
 	goto out;
     }
 
-    rv = fcntl(sd->fd, F_SETFL, O_NONBLOCK);
+    rv = socket_set_nonblock(sd->fd);
     if (rv == -1) {
-	close(sd->fd);
+	close_socket(sd->fd);
 	sd->fd = -1;
-	sd->logchan->log(sd->logchan, OS_ERROR, NULL,
-			 "Error setting nonblock on tcp sol port socket"
-			 " for %s:%s: %s",
-			 sol->tcpdest, sol->tcpport, strerror(errno));
+	sys->log(sys, OS_ERROR, NULL,
+		 "Error setting nonblock on tcp sol port socket"
+		 " for %s:%s: %s",
+		 sol->tcpdest, sol->tcpport, strerror(errno));
 	goto out;
     }
 
@@ -683,7 +689,7 @@ static void sol_set_history_return_size(lmc_data_t    *mc,
 					unsigned int  *rdata_len,
 					void          *cb_data)
 {
-    ipmi_sol_t *sol = ipmi_mc_get_sol(mc);
+    ipmi_sol_t *sol = is_mc_get_sol(mc);
     soldata_t *sd = sol->soldata;
     unsigned int size;
 
@@ -715,7 +721,7 @@ static void sol_get_history_return_size(lmc_data_t    *mc,
 					unsigned int  *rdata_len,
 					void          *cb_data)
 {
-    ipmi_sol_t *sol = ipmi_mc_get_sol(mc);
+    ipmi_sol_t *sol = is_mc_get_sol(mc);
     soldata_t *sd = sol->soldata;
 
     if (!sd) {
@@ -814,14 +820,14 @@ copy_history_buffer(ipmi_sol_t *sol, unsigned int *rsize)
 unsigned char *
 sol_set_frudata(lmc_data_t *mc, unsigned int *size)
 {
-    ipmi_sol_t *sol = ipmi_mc_get_sol(mc);
+    ipmi_sol_t *sol = is_mc_get_sol(mc);
 
     return copy_history_buffer(sol, size);
 }
 
 void sol_free_frudata(lmc_data_t *mc, unsigned char *data)
 {
-    ipmi_sol_t *sol = ipmi_mc_get_sol(mc);
+    ipmi_sol_t *sol = is_mc_get_sol(mc);
     soldata_t *sd = sol->soldata;
 
     if (data)
@@ -835,7 +841,7 @@ ipmi_sol_activate(lmc_data_t    *mc,
 		  unsigned char *rdata,
 		  unsigned int  *rdata_len)
 {
-    ipmi_sol_t *sol = ipmi_mc_get_sol(mc);
+    ipmi_sol_t *sol = is_mc_get_sol(mc);
     soldata_t *sd = sol->soldata;
     uint16_t port;
     int rv;
@@ -953,7 +959,7 @@ ipmi_sol_deactivate(lmc_data_t    *mc,
 		    unsigned char *rdata,
 		    unsigned int  *rdata_len)
 {
-    ipmi_sol_t *sol = ipmi_mc_get_sol(mc);
+    ipmi_sol_t *sol = is_mc_get_sol(mc);
     unsigned int instance;
     uint32_t session_id;
 
@@ -989,10 +995,10 @@ ipmi_sol_deactivate(lmc_data_t    *mc,
 static void
 sol_update_bitrate(lmc_data_t *mc)
 {
-    ipmi_sol_t *sol = ipmi_mc_get_sol(mc);
+    ipmi_sol_t *sol = is_mc_get_sol(mc);
     soldata_t *sd = sol->soldata;
 
-    sd->update_bitrate(ipmi_mc_get_sol(mc));
+    sd->update_bitrate(is_mc_get_sol(mc));
 }
 
 static void
@@ -1417,7 +1423,7 @@ handle_sol_payload(lanserv_data_t *lan, msg_t *msg)
     if (!mc)
 	return;
 
-    sol = ipmi_mc_get_sol(mc);
+    sol = is_mc_get_sol(mc);
     if (msg->sid == sol->session_id)
 	handle_sol_port_payload(lan, sol, msg);
     else if (msg->sid == sol->history_session_id)
@@ -1452,9 +1458,9 @@ sol_write_ready(int fd, void *cb_data)
 
     rv = write(fd, sd->inbuf, sd->inlen);
     if (rv < 0) {
-	sd->logchan->log(sd->logchan, OS_ERROR, NULL,
-			 "Error writing to serial port: %d, disabling\n",
-			 errno);
+	sd->sys->log(sd->sys, OS_ERROR, NULL,
+		     "Error writing to serial port: %d, disabling\n",
+		     errno);
 	sol_port_error(sol);
 	return;
     }
@@ -1596,9 +1602,9 @@ sol_data_ready(int fd, void *cb_data)
 
     rv = read(fd, buf, readsize);
     if (rv < 0) {
-	sd->logchan->log(sd->logchan, OS_ERROR, NULL,
-			 "Error reading from serial port: %d, disabling\n",
-			 errno);
+	sd->sys->log(sd->sys, OS_ERROR, NULL,
+		     "Error reading from serial port: %d, disabling\n",
+		     errno);
 	sol_port_error(sol);
 	return;
     } else if (rv == 0) {
@@ -1637,7 +1643,7 @@ sol_data_ready(int fd, void *cb_data)
 }
 
 int
-sol_read_config(char **tokptr, sys_data_t *sys, const char **err)
+is_sol_read_config(char **tokptr, sys_data_t *sys, const char **err)
 {
     ipmi_sol_t *sol = sys->sol;
     unsigned int val;
@@ -1646,7 +1652,7 @@ sol_read_config(char **tokptr, sys_data_t *sys, const char **err)
 
     sys->sol->use_rtscts = 1;
 
-    rv = get_delim_str(tokptr, &sol->device, err);
+    rv = get_delim_str(sys, tokptr, &sol->device, err);
     if (rv)
 	return rv;
 
@@ -1698,7 +1704,7 @@ sol_read_config(char **tokptr, sys_data_t *sys, const char **err)
 		*end = '\0';
 
 		if (strncmp(opt, "backupfile=", 11) == 0) {
-		    sol->backupfile = strdup(opt + 11);
+		    sol->backupfile = sys_strdup(sys, opt + 11);
 		} else {
 		    *err = "Unknown history option";
 		    return -1;
@@ -1763,7 +1769,7 @@ read_sol_config(sys_data_t *sys)
 
 	if (!mc)
 	    continue;
-	sol = ipmi_mc_get_sol(mc);
+	sol = is_mc_get_sol(mc);
 	if (!sol->configured)
 	    continue;
 
@@ -1772,7 +1778,7 @@ read_sol_config(sys_data_t *sys)
 	sol->solparm.enabled = 1;
 	sol->solparm.bitrate_nonv = 0;
 
-	p = read_persist("sol.mc%2.2x", ipmi_mc_get_ipmb(mc));
+	p = read_persist(sys, "sol.mc%2.2x", sys->mc_get_ipmb(mc));
 	if (p) {
 	    if (!read_persist_int(p, &iv, "enabled"))
 		sol->solparm.enabled = iv;
@@ -1793,18 +1799,19 @@ read_sol_config(sys_data_t *sys)
 }
 
 int
-write_sol_config(lmc_data_t *mc)
+write_sol_config(sys_data_t *sys, lmc_data_t *mc)
 {
     ipmi_sol_t *sol;
     persist_t *p;
 
-    sol = ipmi_mc_get_sol(mc);
+    sol = is_mc_get_sol(mc);
 
-    p = alloc_persist("sol.mc%2.2x", ipmi_mc_get_ipmb(mc));
+    p = alloc_persist(sys, "sol.mc%2.2x",
+		      sol->soldata->sys->mc_get_ipmb(mc));
     if (!p)
 	return ENOMEM;
 
-    sol = ipmi_mc_get_sol(mc);
+    sol = is_mc_get_sol(mc);
 
     add_persist_int(p, sol->solparm.enabled, "enabled");
     add_persist_int(p, sol->solparm.bitrate_nonv, "bitrate");
@@ -1815,17 +1822,17 @@ write_sol_config(lmc_data_t *mc)
 }
 
 int
-sol_init(sys_data_t *sys)
+sol_init(emu_data_t *emu)
 {
     int rv;
 
-    rv = ipmi_emu_register_oi_iana_handler(
+    rv = ipmi_emu_register_oi_iana_handler(emu,
 	OPENIPMI_IANA_CMD_SET_HISTORY_RETURN_SIZE,
 	sol_set_history_return_size, NULL);
     if (rv)
 	return rv;
 
-    rv = ipmi_emu_register_oi_iana_handler(
+    rv = ipmi_emu_register_oi_iana_handler(emu,
 	OPENIPMI_IANA_CMD_GET_HISTORY_RETURN_SIZE,
 	sol_get_history_return_size, NULL);
     if (rv)
@@ -1870,16 +1877,15 @@ handle_sol_shutdown(void *info, int sig)
     fclose(f);
 }
 
-int
+static int
 sol_init_mc(sys_data_t *sys, lmc_data_t *mc)
 {
-    ipmi_sol_t *sol = ipmi_mc_get_sol(mc);
+    ipmi_sol_t *sol = is_mc_get_sol(mc);
     soldata_t *sd;
 
     sd = sys->alloc(sys, sizeof(*sd));
     if (!sd)
 	return ENOMEM;
-    memset(sd, 0, sizeof(*sd));
     sd->sys = sys;
     sol->soldata = sd;
 
@@ -1928,7 +1934,7 @@ sol_init_mc(sys_data_t *sys, lmc_data_t *mc)
     sd->fd = -1;
     sd->curr_packet_seq = 1;
     sd->history_curr_packet_seq = 1;
-    sd->logchan = ipmi_mc_get_channelset(mc)[0];
+    sd->logchan = sd->sys->mc_get_channelset(mc)[0];
 
     if (sol_port_init(sol)) {
 	/* Retry in 10 seconds. */
