@@ -886,6 +886,7 @@ struct ipmi_lan_config_s
     unsigned char  max_priv_for_cipher_suite_supported;
     unsigned char  max_priv_for_cipher_suite[16];
 
+    /* See the note in the gnd function foro weirdness about this field. */
     unsigned char num_alert_destinations;
     unsigned char  vlan_tag_supported;
     alert_dest_type_t *alert_dest_type;
@@ -1095,6 +1096,12 @@ static int gnd(ipmi_lan_config_t *lanc, lanparms_t *lp, int err,
     if (num == 0)
 	return 0;
 
+    /*
+     * This is important!  The number in this field is the number of
+     * non-volatile destinations.  There is a volatile destination
+     * at zero that is always present, and at least on non-volatile
+     * field is required if this paramter is non-zero.
+     */
     num++;
 
     lanc->alert_dest_type = ipmi_mem_alloc(sizeof(alert_dest_type_t) * num);
@@ -1554,7 +1561,7 @@ got_parm(ipmi_lanparm_t    *lanparm,
 	    if (lanc->num_alert_destinations == 0)
 		goto done;
 	    lanc->curr_parm = IPMI_LANPARM_DEST_VLAN_TAG;
-	    lanc->curr_sel = 1;
+	    lanc->curr_sel = 0;
 	}
 	break;
 
@@ -1766,7 +1773,7 @@ set_done(ipmi_lanparm_t *lanparm,
 	 void           *cb_data)
 {
     ipmi_lan_config_t *lanc = cb_data;
-    unsigned char     data[MAX_IPMI_DATA_SIZE];
+    unsigned char     data[MAX_IPMI_DATA_SIZE] = { 0, 0 };
     lanparms_t        *lp = &(lanparms[lanc->curr_parm]);
 
     if (err == IPMI_IPMI_ERR_VAL(0x82)) {
@@ -1784,6 +1791,21 @@ set_done(ipmi_lanparm_t *lanparm,
 
  next_parm:
     switch (lanc->curr_parm) {
+    /*
+     * Set the IP address source before the IP address itself.  Some
+     * BMCs are picky and won't set the IP address if the address
+     * source is not static.
+     */
+    case IPMI_LANPARM_AUTH_TYPE_ENABLES:
+	lanc->curr_parm = IPMI_LANPARM_IP_ADDRESS_SRC;
+	break;
+    case IPMI_LANPARM_IP_ADDRESS_SRC:
+	lanc->curr_parm = IPMI_LANPARM_IP_ADDRESS;
+	break;
+    case IPMI_LANPARM_IP_ADDRESS:
+	lanc->curr_parm = IPMI_LANPARM_MAC_ADDRESS;
+	break;
+
     case IPMI_LANPARM_NUM_DESTINATIONS:
 	lanc->curr_parm++;
 	if (lanc->num_alert_destinations == 0) {
@@ -1840,6 +1862,22 @@ set_done(ipmi_lanparm_t *lanparm,
 	goto next_parm;
     }
 
+    if ((lanc->ip_addr_source == IPMI_LANPARM_IP_ADDR_SRC_DHCP) &&
+	(lanc->curr_parm == IPMI_LANPARM_IP_ADDRESS ||
+	 lanc->curr_parm == IPMI_LANPARM_SUBNET_MASK ||
+	 lanc->curr_parm == IPMI_LANPARM_DEFAULT_GATEWAY_ADDR ||
+	 lanc->curr_parm == IPMI_LANPARM_BACKUP_GATEWAY_ADDR))
+    {
+	/*
+	 * Don't set the fields that come from DHCP if the address
+	 * source is DHCP.  Some BMCs are picky about this and will
+	 * error if you try to set these fields and the address source
+	 * is DHCP.
+	 */
+	goto next_parm;
+    }
+
+
     lp->set_handler(lanc, lp, data);
     err = ipmi_lanparm_set_parm(lanparm, lanc->curr_parm,
 				data, lp->length, set_done, lanc);
